@@ -210,7 +210,7 @@ GoAccess.Util = {
 	},
 
 	// Format field value to human readable
-	fmtValue: function (value, dataType, decimals, shorten) {
+	fmtValue: function (value, dataType, decimals, shorten, hlregex, hlvalue) {
 		var val = 0;
 		if (!dataType)
 			val = value;
@@ -230,7 +230,7 @@ GoAccess.Util = {
 			val = this.formatBytes(value, decimals);
 			break;
 		case 'percent':
-			val = parseFloat(value.replace(',', '.')).toFixed(2) + '%';
+			val = value.replace(',', '.') + '%';
 			break;
 		case 'time':
 			if (this.isNumeric(value))
@@ -245,6 +245,18 @@ GoAccess.Util = {
 			val = value;
 		}
 
+		if (hlregex) {
+			let o = JSON.parse(hlregex), tmp = '';
+			for (var x in o) {
+				tmp = val.replace(new RegExp(x, 'gi'), o[x]);
+				if (tmp != val) {
+					val = tmp;
+					break;
+				}
+				val = tmp;
+			}
+		}
+
 		return value == 0 ? String(val) : (val === undefined ? '—' : val);
 	},
 
@@ -337,6 +349,8 @@ GoAccess.Util = {
 
 // OVERALL STATS
 GoAccess.OverallStats = {
+	total_requests: 0,
+
 	// Render each overall stats box
 	renderBox: function (data, ui, row, x, idx) {
 		var wrap = $('.wrap-general-items');
@@ -388,6 +402,7 @@ GoAccess.OverallStats = {
 	initialize: function () {
 		var ui = GoAccess.getPanelUI('general');
 		var data = GoAccess.getPanelData('general');
+		this.total_requests = data.total_requests;
 
 		this.renderData(data, ui);
 	}
@@ -585,6 +600,7 @@ GoAccess.Nav = {
 		case 'status_codes'    : return 'warning';
 		case 'remote_user'     : return 'users';
 		case 'geolocation'     : return 'map-marker';
+		case 'asn'             : return 'map-marker';
 		case 'mime_type'       : return 'file-o';
 		case 'tls_type'        : return 'warning';
 		default                : return 'pie-chart';
@@ -1466,7 +1482,7 @@ GoAccess.Tables = {
 		// use metaType if exist else fallback to dataType
 		var vtype = ui.metaType || ui.dataType;
 		var className = ui.className || '';
-		className += ui.dataType != 'string' ? 'text-right' : '';
+		className += !['string'].includes(ui.dataType) ? 'text-right' : '';
 		return {
 			'className': className,
 			'value'    : val ? GoAccess.Util.fmtValue(val, vtype) : null,
@@ -1547,11 +1563,11 @@ GoAccess.Tables = {
 	// e.g., value = Object {count: 14351, percent: 5.79}
 	getObjectCell: function (panel, ui, value) {
 		var className = ui.className || '';
-		className += ui.dataType != 'string' ? 'text-right' : '';
+		className += !['string'].includes(ui.dataType) ? 'text-right' : '';
 		return {
 			'className': className,
 			'percent': GoAccess.Util.getPercent(value),
-			'value': GoAccess.Util.fmtValue(GoAccess.Util.getCount(value), ui.dataType)
+			'value': GoAccess.Util.fmtValue(GoAccess.Util.getCount(value), ui.dataType, null, null, ui.hlregex, ui.hlvalue, ui.hlidx)
 		};
 	},
 
@@ -1841,6 +1857,13 @@ GoAccess.App = {
 		if (!this.hasFocus)
 			return;
 
+		// some panels may not have been properly rendered since no data was
+		// passed when bootstrapping the report, thus we do a one full
+		// re-render of all panels
+		if (GoAccess.OverallStats.total_requests == 0 && GoAccess.OverallStats.total_requests != GoAccess.AppData.general.total_requests)
+			GoAccess.Panels.initialize();
+		GoAccess.OverallStats.total_requests = GoAccess.AppData.general.total_requests;
+
 		this.verifySort();
 		GoAccess.OverallStats.initialize();
 
