@@ -34,6 +34,7 @@
 #include <string.h>
 #include <stddef.h>
 
+#include "opesys.h"
 #include "browsers.h"
 
 #include "error.h"
@@ -403,17 +404,34 @@ parse_browsers_file (void) {
  * If it is a crawler, 1 is returned . */
 int
 is_crawler (const char *agent) {
-  char type[BROWSER_TYPE_LEN];
-  char *browser, *a;
+  char btype[BROWSER_TYPE_LEN];
+  char otype[OPESYS_TYPE_LEN];
+  char *browser, *os, *a;
 
   if (agent == NULL || *agent == '\0')
     return 0;
 
-  if ((a = xstrdup (agent), browser = verify_browser (a, type)) != NULL)
+  if ((a = xstrdup (agent), browser = verify_browser (a, btype)) != NULL)
     free (browser);
   free (a);
 
-  return strcmp (type, "Crawlers") == 0 ? 1 : 0;
+  if (strcmp (btype, "Crawlers") == 0)
+    return 1;
+
+  if (!conf.unknowns_as_crawlers)
+    return 0;
+
+  if (strcmp (btype, "Unknown") == 0)
+    return 1;
+
+  if ((a = xstrdup (agent), os = verify_os (a, otype)) != NULL)
+    free (os);
+  free (a);
+
+  if (strcmp (otype, "Unknown") == 0)
+    return 1;
+
+  return 0;
 }
 
 /* Return the Opera 15 and beyond.
@@ -553,9 +571,6 @@ verify_browser (char *str, char *type) {
   if (str == NULL || *str == '\0')
     return NULL;
 
-  if ((match = check_http_crawler (str)) && (token = parse_crawler (str, match, type)))
-    return token;
-
   /* check user's list */
   for (i = 0; i < conf.browsers_hash_idx; ++i) {
     if ((match = strstr (str, conf.user_browsers_hash[i][0])) == NULL)
@@ -563,6 +578,10 @@ verify_browser (char *str, char *type) {
     return parse_browser (match, type, i, conf.user_browsers_hash);
   }
 
+  /* try heuristics */
+  if ((match = check_http_crawler (str)) && (token = parse_crawler (str, match, type)))
+    return token;
+
   /* fallback to default browser list */
   for (j = 0; j < ARRAY_SIZE (browsers); ++j) {
     if ((match = strstr (str, browsers_hash[j][0])) == NULL)
