@@ -43,7 +43,7 @@
 #define __attribute__(x) /**/
 #endif
 #define GO_UNUSED __attribute__((unused))
-#define GO_VERSION 		"1.6.5"
+#define GO_VERSION 		"1.7"
 #define GO_WEBSITE 		"https://goaccess.io/"
 extern struct tm now_tm;
 
@@ -52,7 +52,7 @@ extern struct tm now_tm;
 
 /* total number of modules */
 #ifdef HAVE_GEOLOCATION
-#define TOTAL_MODULES    18
+#define TOTAL_MODULES    19
 #else
 #define TOTAL_MODULES    17
 #endif
@@ -109,6 +109,7 @@ typedef enum MODULES {
   CACHE_STATUS,
 #ifdef HAVE_GEOLOCATION
   GEO_LOCATION,
+  ASN,
 #endif
   MIME_TYPE,
   TLS_TYPE,
