@@ -123,6 +123,7 @@ static GScroll gscroll = {
      {0, 0}, /* CACHE_STATUS    { scroll, offset} */
 #ifdef HAVE_GEOLOCATION
      {0, 0}, /* GEO_LOCATION    { scroll, offset} */
+     {0, 0}, /* ASN             { scroll, offset} */
 #endif
      {0, 0}, /* MIME_TYPE       { scroll, offset} */
      {0, 0}, /* TLS_TYPE        { scroll, offset} */
@@ -498,8 +499,12 @@ load_ip_agent_list (void) {
   int type_ip = 0;
   /* make sure we have a valid IP */
   int sel = gscroll.module[gscroll.current].scroll;
-  GDashData item = dash->module[HOSTS].data[sel];
+  GDashData item = { 0 };
 
+  if (dash->module[HOSTS].holder_size == 0)
+    return;
+
+  item = dash->module[HOSTS].data[sel];
   if (!invalid_ipaddr (item.metrics->data, &type_ip))
     load_agent_list (main_win, item.metrics->data);
 }
@@ -707,7 +712,7 @@ tail_html (void) {
   allocate_holder ();
 
   pthread_mutex_lock (&gdns_thread.mutex);
-  json = get_json (holder, 0);
+  json = get_json (holder, 1);
   pthread_mutex_unlock (&gdns_thread.mutex);
 
   if (json == NULL)
@@ -725,7 +730,7 @@ fast_forward_client (int listener) {
   char *json = NULL;
 
   pthread_mutex_lock (&gdns_thread.mutex);
-  json = get_json (holder, 0);
+  json = get_json (holder, 1);
   pthread_mutex_unlock (&gdns_thread.mutex);
 
   if (json == NULL)
@@ -1110,13 +1115,18 @@ get_keys (Logs * logs) {
       if (set_module_to (&gscroll, GEO_LOCATION) == 0)
         render_screens (offset);
       break;
-#endif
     case 38:   /* Shift + 7 */
+      /* reset expanded module */
+      if (set_module_to (&gscroll, ASN) == 0)
+        render_screens (offset);
+      break;
+#endif
+    case 42:   /* Shift + 7 */
       /* reset expanded module */
       if (set_module_to (&gscroll, MIME_TYPE) == 0)
         render_screens (offset);
       break;
-    case 42:   /* Shift + 8 */
+    case 40:   /* Shift + 8 */
       /* reset expanded module */
       if (set_module_to (&gscroll, TLS_TYPE) == 0)
         render_screens (offset);
