@@ -87,6 +87,7 @@
 #define MTRC_PROTOCOLS_LBL       _( "Protocol")
 #define MTRC_PROTOCOLS_SHORT_LBL _( "Proto")
 #define MTRC_CITY_LBL            _( "City")
+#define MTRC_ASB_LBL             _( "ASN")
 #define MTRC_COUNTRY_LBL         _( "Country")
 #define MTRC_HOSTNAME_LBL        _( "Hostname")
 #define MTRC_DATA_LBL            _( "Data")
@@ -201,6 +202,13 @@
 #define GEO_LOCATION_LABEL             \
   N_("Geo Location")
 
+#define ASN_HEAD                 \
+  N_("ASN")
+#define ASN_DESC                 \
+  N_("Autonomous System Numbers/Organizations (ASNs)")
+#define ASN_LABEL                \
+  N_("ASN")
+
 #define STATUS_CODES_HEAD              \
   N_("HTTP Status Codes")
 #define STATUS_CODES_DESC              \
