@@ -133,6 +133,10 @@ static GHTML htmldef[] = {
     {CHART_VBAR, hits_visitors_plot, 0, 1, NULL, NULL},
     {CHART_VBAR, hits_bw_plot, 0, 1, NULL, NULL},
   }},
+  {ASN, 1, print_metrics, {
+    {CHART_VBAR, hits_visitors_plot, 0, 0, NULL, NULL},
+    {CHART_VBAR, hits_bw_plot, 0, 0, NULL, NULL},
+  }},
 #endif
   {MIME_TYPE, 1, print_metrics, {
     {CHART_VBAR, hits_visitors_plot, 0, 1, NULL, NULL},
@@ -535,6 +539,8 @@ print_def_metric (FILE * fp, const GDefMetric def, int sp) {
     fpskeysval (fp, "metaLabel", def.metalbl, isp, 0);
   if (def.datatype)
     fpskeysval (fp, "dataType", def.datatype, isp, 0);
+  if (def.hlregex)
+    fpskeysval (fp, "hlregex", def.hlregex, isp, 0);
   if (def.datakey)
     fpskeysval (fp, "key", def.datakey, isp, 0);
   if (def.lbl)
@@ -799,6 +805,11 @@ print_def_method (FILE * fp, int sp) {
     .lbl = MTRC_METHODS_LBL,
     .datatype = "string",
     .cwidth = "6%",
+    .hlregex = "{"
+      "\\\"(\\\\\\\\b[A-Z]{3}\\\\\\\\b)\\\": \\\"<b class='cell-hl b1'>$1</b>\\\","
+      "\\\"(\\\\\\\\b[A-Z]{4}\\\\\\\\b)\\\": \\\"<b class='cell-hl b2'>$1</b>\\\","
+      "\\\"(\\\\\\\\b[A-Z]{5,}\\\\\\\\b)\\\": \\\"<b class='cell-hl b3'>$1</b>\\\"" "}",
+
   };
 
   if (!conf.append_method)
@@ -815,6 +826,11 @@ print_def_protocol (FILE * fp, int sp) {
     .lbl = MTRC_PROTOCOLS_LBL,
     .datatype = "string",
     .cwidth = "7%",
+    .hlregex = "{"
+      "\\\"(\\\\\\\\bHTTP/1.0\\\\\\\\b)\\\": \\\"<b class='cell-hl d1'>$1</b>\\\","
+      "\\\"(\\\\\\\\bHTTP/1.1\\\\\\\\b)\\\": \\\"<b class='cell-hl d2'>$1</b>\\\","
+      "\\\"(\\\\\\\\bHTTP/2\\\\\\\\b)\\\": \\\"<b class='cell-hl d3'>$1</b>\\\","
+      "\\\"(\\\\\\\\bHTTP/3\\\\\\\\b)\\\": \\\"<b class='cell-hl d4'>$1</b>\\\"" "}",
   };
 
   if (!conf.append_protocol)
@@ -838,6 +854,24 @@ print_def_city (FILE * fp, int sp) {
   print_def_block (fp, def, sp, 0);
 }
 
+/* Output JSON ASN definition block. */
+static void
+print_def_asn (FILE * fp, int sp) {
+  GDefMetric def = {
+    .datakey = "asn",
+    .lbl = MTRC_ASB_LBL,
+    .datatype = "string",
+    .hlregex = "{"
+      "\\\"^(\\\\\\\\d+)\\\": \\\"<b>$1</b>\\\","
+      "\\\"^(AS\\\\\\\\d+)\\\": \\\"<b>$1</b>\\\"" "}",
+  };
+
+  if (!conf.has_geoasn)
+    return;
+
+  print_def_block (fp, def, sp, 0);
+}
+
 /* Output JSON country definition block. */
 static void
 print_def_country (FILE * fp, int sp) {
@@ -845,6 +879,7 @@ print_def_country (FILE * fp, int sp) {
     .datakey = "country",
     .lbl = MTRC_COUNTRY_LBL,
     .datatype = "string",
+    .hlregex = "{" "\\\"^([A-Z]{2})\\\": \\\"<b class='span-hl g5'>$1</b>\\\"" "}",
   };
 
   if (!conf.has_geocountry)
@@ -881,6 +916,12 @@ print_def_data (FILE * fp, GModule module, int sp) {
     .metakey = "unique",
     .metalbl = "Total",
     .metatype = "numeric",
+    .hlregex = "{" "\\\"^(\\\\\\\\d+|\\\\\\\\d+xx)(\\\\\\\\s.*)$\\\": \\\"<b>$1</b>$2\\\","     /* 2xx Success */
+      "\\\"^(AS\\\\\\\\d+)\\\": \\\"<b>$1</b>\\\","     /* AS9823 Google */
+      "\\\"^(\\\\\\\\d+:)\\\": \\\"<b>$1</b>\\\","      /* 01234: Data */
+      "\\\"(\\\\\\\\d+)|(:\\\\\\\\d+)|(:\\\\\\\\d+:\\\\\\\\d+)\\\": \\\"$1<b>$2</b>\\\","       /* 12/May/2022:12:34 */
+      "\\\"^([A-Z]{2})(\\\\\\\\s.*$)\\\": \\\"<b class='span-hl g5'>$1</b>$2\\\""       /* US United States */
+      "}",
   };
 
   print_def_block (fp, def, sp, 1);
@@ -934,6 +975,7 @@ print_host_metrics (FILE * fp, const GHTML * def, int sp) {
 
   print_def_city (fp, sp);
   print_def_country (fp, sp);
+  print_def_asn (fp, sp);
   print_def_hostname (fp, sp);
 
   print_def_data (fp, def->module, sp);
