@@ -79,6 +79,8 @@ const int sort_choices[][SORT_MAX_OPTS] = {
 #ifdef HAVE_GEOLOCATION
   /* GEO_LOCATION */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* ASN */
+  {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
 #endif
   /* MIME_TYPE */
   {SORT_BY_HITS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
@@ -121,6 +123,7 @@ GSort module_sort[TOTAL_MODULES] = {
   {CACHE_STATUS        , SORT_BY_HITS , SORT_DESC } ,
 #ifdef HAVE_GEOLOCATION
   {GEO_LOCATION        , SORT_BY_HITS , SORT_DESC } ,
+  {ASN                 , SORT_BY_HITS , SORT_DESC } ,
 #endif
   {MIME_TYPE           , SORT_BY_HITS , SORT_DESC } ,
   {TLS_TYPE            , SORT_BY_VISITORS , SORT_DESC } ,
