@@ -46,6 +46,7 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <inttypes.h>
+#include <regex.h>
 
 #include <sys/types.h>
 #include <netinet/in.h>
@@ -255,6 +256,56 @@ wc_match (const char *wc, char *str) {
   return 0;
 }
 
+/* Generic routine to extract all groups from a string given a POSIX regex.
+ *
+ * If no match found or error, NULL is returned.
+ * If match found, a string is returned. */
+char *
+regex_extract_string (const char *str, const char *regex, int max_groups, char const **err) {
+  char *copy = NULL, *dest = NULL;
+  int i, ret = 0;
+  regex_t re;
+  regmatch_t groups[max_groups];
+
+  if (str == NULL || *str == '\0') {
+    *err = "Invalid string.";
+    return NULL;
+  }
+
+  if (regcomp (&re, regex, REG_EXTENDED)) {
+    *err = "Unable to compile regular expression upon extraction";
+    return NULL;
+  }
+
+  ret = regexec (&re, str, max_groups, groups, 0);
+  if (ret == REG_NOMATCH) {
+    *err = "Unable to match regular expression extraction.";
+    goto out;
+  }
+
+  if (ret != 0) {
+    *err = "Error while matching regular expression extraction.";
+    goto out;
+  }
+
+  dest = xstrdup ("");
+  for (i = 0; i < max_groups; ++i) {
+    if (groups[i].rm_so == -1)
+      break;
+
+    copy = xstrdup (str);
+    copy[groups[i].rm_eo] = 0;
+
+    append_str (&dest, copy + groups[i].rm_so);
+    free (copy);
+  }
+
+out:
+  regfree (&re);
+
+  return ret == 0 ? dest : NULL;
+}
+
 /* Determine if the given host needs to be ignored given the list of
  * referrers to ignore.
  *
